/*
** Copyright (C) 2001-2018 by Carnegie Mellon University.
**
** @OPENSOURCE_LICENSE_START@
** See license information in ../../LICENSE.txt
** @OPENSOURCE_LICENSE_END@
*/

/*
**  utils.h
**
**    A collection of utility routines.
**
**    Suresh L Konda
*/
#ifndef _UTILS_H
#define _UTILS_H
#ifdef __cplusplus
extern "C" {
#endif

#include <silk-ipset/silk-ipset.h>

RCSIDENTVAR(rcsID_UTILS_H, "$SiLK: skutils.h e85db1eebd9f 2018-12-13 18:33:20Z mthomas $");

#ifdef SK_HAVE_GETOPT_LONG_ONLY
#include <getopt.h>
#else
#include <silk-ipset/gnu_getopt.h>
#endif

/**
 *  @file
 *
 *    A collection of utility functions for registring an application,
 *    registering options, providing common options, parsing option
 *    strings, printing error messages, opening and closing file, et
 *    cetera.
 *
 *    This file is part of libskipset.
 */


/**
 *    Where to create temp files by default.  This can be overridden
 *    by the --temp-dir switch (assuming skOptionsTempDirRegister() is
 *    in use), or environment variable named in SK_TEMPDIR_ENVAR1 or
 *    the environment variable named in SK_TEMPDIR_ENVAR2.
 *
 *    If this is not defined, no default exists.
 */
#define SK_TEMPDIR_DEFAULT "/tmp"

/**
 *    Name of primary environment variable that holds name of temp
 *    directory.  This is consulted when the --temp-dir switch is not
 *    given.
 */
#define SK_TEMPDIR_ENVAR1 "SILK_TMPDIR"

/**
 *    Name of alternate environment variable that holds name of temp
 *    directory.  Used when the --temp-dir switch is not given and the
 *    variable named by SK_TMPDIR_ENVAR1 is not set.
 */
#define SK_TEMPDIR_ENVAR2 "TMPDIR"



typedef enum {
    /* Command was successful */
    SKUTILS_OK = 0,

    /* Input to function is null or invalid (e.g., 0 length bitmap) */
    SKUTILS_ERR_INVALID = -1,

    /* Input to function was empty (or contained only whitespace) */
    SKUTILS_ERR_EMPTY = -2,

    /* Unexpected/Bad character or number is unparseable */
    SKUTILS_ERR_BAD_CHAR = -3,

    /* Value overflows the parser */
    SKUTILS_ERR_OVERFLOW = -4,

    /* Value underflows the parser */
    SKUTILS_ERR_UNDERFLOW = -5,

    /* Range is invalid (min > max) */
    SKUTILS_ERR_BAD_RANGE = -6,

    /* Unexpected end-of-input */
    SKUTILS_ERR_SHORT = -7,

    /* Too many fields provided */
    SKUTILS_ERR_TOO_MANY_FIELDS = -8,

    /* Out of memory */
    SKUTILS_ERR_ALLOC = -9,

    /* Miscellaneous error */
    SKUTILS_ERR_OTHER = -10,

    /* Value is below the minimum */
    SKUTILS_ERR_MINIMUM = -11,

    /* Value is above the maximum */
    SKUTILS_ERR_MAXIMUM = -12,

    /* Host name or port could not be resolved */
    SKUTILS_ERR_RESOLVE = -13
} silk_utils_errcode_t;



/*
**
**  sku-compat.c
**
*/

/* silk.h will #define intmax_t sk_intmax_t if required */
typedef int64_t sk_intmax_t;

/* silk.h will #define imaxdiv_t sk_imaxdiv_t if required */
typedef struct sk_imaxdiv_st {
    sk_intmax_t quot; /* Quotient  */
    sk_intmax_t rem;  /* Remainder */
} sk_imaxdiv_t;

/* silk.h will #define imaxdiv sk_imaxdiv if required */
/**
 *    Do the integer division of 'numer' by 'denom'; return a
 *    structure that contains the 'quot' (quotient) and the 'rem'
 *    (remainder).
 *
 *    Conforming to C99.
 */
sk_imaxdiv_t
sk_imaxdiv(
    sk_intmax_t         numer,
    sk_intmax_t         denom);


/* silk.h will #define memccpy sk_memccpy if required */
/**
 *    Copy bytes from 'src' to 'dst' stopping after the first
 *    occurance of 'c' or when 'len' octets have been copied,
 *    whichever comes first.  If 'c' was not found, NULL is returned;
 *    else return value pointing to character after 'c' in 'dst'.
 *
 *    Conforming to C99.
 */
void *
sk_memccpy(
    void               *dst,
    const void         *src,
    int                 c,
    size_t              len);


/* silk.h will #define setenv sk_setenv if required */
/**
 *    Set environment variable 'name' to 'value', unless 'name' already
 *    exists in the environment and 'overwrite' is 0.
 *
 *    Return 0 on success.  Return -1 and set errno on memory
 *    allocation error or if 'name' contains a '=' character.  Assumes
 *    'name' and 'value' are not NULL.
 *
 *    Conforming to POSIX.1-2001
 */
int
sk_setenv(
    const char         *name,
    const char         *value,
    int                 overwrite);


/* silk.h will #define strsep sk_strsep if required */
/**
 *    Locate in *stringp first occurrence of any member of 'delim'.
 *    Change that character to '\0', modify '*stringp' to point to the
 *    following character, and return the original value of *stringp.
 *
 *    Return NULL if *stringp is NULL or when **stringp is '\0'; i.e.,
 *    when the end of the string is reached.
 *
 *    Empty field is determined by testing the returned value to '\0'.
 *
 *    Comforming to BSD 4.4
 */
char *
sk_strsep(
    char              **stringp,
    const char         *delim);


/*
**
**  skbitmap.c
**
*/


/* typedef struct sk_bitmap_st sk_bitmap_t; // silk_types.h */
struct sk_bitmap_st {
    uint32_t   *map;
    uint32_t    num_bits;
    uint32_t    count;
};


/**
 *    Type for iterating over the entries in an sk_bitmap_t.
 */
typedef struct sk_bitmap_iter_st {
    const sk_bitmap_t  *bitmap;
    uint32_t            map_idx;
    uint8_t             pos;
} sk_bitmap_iter_t;


/* Internal macros.  Not for public use. */
#define _BMAP_INDEX(p)      ((p) >> 5)
#define _BMAP_OFFSET(p)     (1u << ((p) & 0x1F))
#define _BMAP_IS_SET(b, p)  ((b)->map[_BMAP_INDEX(p)] & _BMAP_OFFSET(p))



/**
 *    Create a new empty bitmap capable of holding 'num_bits' bits.
 *    All bits in the map are set to 0/LOW/OFF.  Return the bitmap in
 *    the location given by 'bitmap_out'.  Returns 0 if successful; -1
 *    for a memory allocation error or if 'num_bits' is zero.
 *
 *    Bits in an sk_bitmap_t are numbered from 0 to num_bits-1.
 *
 *    See also skBitmapBind().
 */
int
skBitmapCreate(
    sk_bitmap_t       **bitmap_out,
    uint32_t            num_bits);


/**
 *    Bind 'bitmap' to an existing array of uint32_t, 'bitarray', and
 *    clear the bitmap.  'num_bits' is number of bits you want to the
 *    bitmap to hold.  'sizeof_bitarray' should be the value of
 *    sizeof(bitarray).
 *
 *    This function can be used to create bitmaps on the stack without
 *    having to use the allocation code in skBitmapCreate().  These
 *    bitmaps should NOT be skBitmapDestroy()'ed.
 */
int
skBitmapBind(
    sk_bitmap_t        *bitmap,
    uint32_t            num_bits,
    uint32_t           *bitarray,
    size_t              sizeof_bitarray);


/**
 *    Destroy the bitmap at location given by 'bitmap' that was
 *    created by skBitmapCreate().  Do not call this function when you
 *    have used skBitmapBind().
 *
 *    The 'bitmap' paramter may be NULL or it may reference a NULL
 *    value.
 */
void
skBitmapDestroy(
    sk_bitmap_t       **bitmap);


/**
 *    Turn OFF all the bits in 'bitmap' and set the high-bit count to
 *    zero.
 */
void
skBitmapClearAllBits(
    sk_bitmap_t        *bitmap);


/**
 *    Turn ON all the bits in 'bitmap' and set the high-bit count to
 *    the number of bits in the bitmap.
 */
void
skBitmapSetAllBits(
    sk_bitmap_t        *bitmap);


/**
 *    Return the number of bits that 'bitmap' can hold.
 */
uint32_t
skBitmapGetSizeF(
    const sk_bitmap_t  *bitmap);
#define skBitmapGetSizeFast(bmap_size_fast)     \
    ((bmap_size_fast)->num_bits)
#ifdef SKBITMAP_DEBUG
#  define skBitmapGetSize(bitmap) skBitmapGetSizeF(bitmap)
#else
#  define skBitmapGetSize(bitmap) skBitmapGetSizeFast(bitmap)
#endif


/**
 *    Return the number of bits in 'bitmap' that are ON.
 */
uint32_t
skBitmapGetHighCountF(
    const sk_bitmap_t  *bitmap);
#define skBitmapGetHighCountFast(bmap_high_fast)        \
    ((bmap_high_fast)->count)
#ifdef SKBITMAP_DEBUG
#  define skBitmapGetHighCount(bitmap) skBitmapGetHighCountF(bitmap)
#else
#  define skBitmapGetHighCount(bitmap) skBitmapGetHighCountFast(bitmap)
#endif


/**
 *    Return 1 if the bit at position 'pos' in 'bitmap' is ON; return
 *    0 if it is OFF.  Valid values for 'pos' are 0 through one less
 *    than the number of bits in the bitmap.  Return -1 if 'pos' is
 *    larger than the size of the bitmap.
 */
int
skBitmapGetBitF(
    const sk_bitmap_t  *bitmap,
    uint32_t            pos);
#define skBitmapGetBitFast(bmap_get_fast, pos_get_fast)         \
    ((((uint32_t)(pos_get_fast)) >= (bmap_get_fast)->num_bits)  \
     ? -1                                                       \
     : !!_BMAP_IS_SET((bmap_get_fast), (pos_get_fast)))
#ifdef SKBITMAP_DEBUG
#  define skBitmapGetBit(bitmap, pos) skBitmapGetBitF(bitmap, pos)
#else
#  define skBitmapGetBit(bitmap, pos) skBitmapGetBitFast(bitmap, pos)
#endif


/**
 *    Turn ON the bit at position 'pos' in 'bitmap'.  Adjust the
 *    bitmap's high-bit counter.  Valid values for 'pos' are 0 through
 *    one less than the number of bits in the bitmap.
 */
void
skBitmapSetBitF(
    sk_bitmap_t        *bitmap,
    uint32_t            pos);
#define skBitmapSetBitFast(bmap_set_fast, pos_set_fast)                 \
    if ((((uint32_t)(pos_set_fast)) >= (bmap_set_fast)->num_bits)       \
        || _BMAP_IS_SET((bmap_set_fast), (pos_set_fast)))               \
    { /* no-op */ } else {                                              \
        (bmap_set_fast)->map[_BMAP_INDEX(pos_set_fast)]                 \
            |= _BMAP_OFFSET(pos_set_fast);                              \
        ++(bmap_set_fast)->count;                                       \
    }
#ifdef SKBITMAP_DEBUG
#  define skBitmapSetBit(bitmap, pos) skBitmapSetBitF(bitmap, pos)
#else
#  define skBitmapSetBit(bitmap, pos) skBitmapSetBitFast(bitmap, pos)
#endif


/**
 *    Turn OFF the bit at position 'pos' in 'bitmap'.  Adjust the
 *    bitmap's high-bit counter.  Valid values for 'pos' are 0 through
 *    one less than the number of bits in the bitmap.
 */
void
skBitmapClearBitF(
    sk_bitmap_t        *bitmap,
    uint32_t            pos);
#define skBitmapClearBitFast(bmap_clear_fast, pos_clear_fast)   \
    if (((pos_clear_fast) >= (bmap_clear_fast)->num_bits)       \
        || !_BMAP_IS_SET((bmap_clear_fast),(pos_clear_fast)))   \
    { /* no-op */ } else {                                      \
        (bmap_clear_fast)->map[_BMAP_INDEX(pos_clear_fast)]     \
            &= ~(_BMAP_OFFSET(pos_clear_fast));                 \
        --(bmap_clear_fast)->count;                             \
    }
#ifdef SKBITMAP_DEBUG
#  define skBitmapClearBit(bitmap, pos) skBitmapClearBitF(bitmap, pos)
#else
#  define skBitmapClearBit(bitmap, pos) skBitmapClearBitFast(bitmap, pos)
#endif


/**
 *    Modify 'bitmap' in place to hold its complement.
 */
void
skBitmapComplement(
    sk_bitmap_t        *bitmap);


/**
 *    Compute the intersection of 'dest' and 'src', placing the result
 *    in 'dest'.  The bitmaps must be of the same size.  If they are
 *    not of the same size, returns -1, else returns 0.
 */
int
skBitmapIntersection(
    sk_bitmap_t        *dest,
    const sk_bitmap_t  *src);


/**
 *    Compute the union of 'dest' and 'src', placing the result in
 *    'dest'.  The bitmaps must be of the same size.  If they are not
 *    of the same size, returns -1, else returns 0.
 */
int
skBitmapUnion(
    sk_bitmap_t        *dest,
    const sk_bitmap_t  *src);


/**
 *    Return the number of bits, starting at 'begin_pos', that have
 *    the value 'state', which should be 0 or 1.  Return UINT32_MAX if
 *    'begin_pos' is larger than the size of the bitmap.
 */
uint32_t
skBitmapCountConsecutive(
    const sk_bitmap_t  *bitmap,
    uint32_t            begin_pos,
    uint32_t            state);


/**
 *    Turn ON all bits in the bitmap from 'begin_pos' to 'end_pos'
 *    inclusive.  Valid values for 'begin_pos' and 'end_pos' are 0
 *    through one less than the number of bits in the bitmap.  Return
 *    -1 if 'end_pos' is less than 'begin_pos' or if either value is
 *    larger than the size of the bitmap.  Return 0 otherwise.
 */
int
skBitmapRangeSet(
    sk_bitmap_t        *bitmap,
    uint32_t            begin_pos,
    uint32_t            end_pos);


/**
 *    Turn OFF all bits in the bitmap from 'begin_pos' to 'end_pos'
 *    inclusive.  Valid values for 'begin_pos' and 'end_pos' are 0
 *    through one less than the number of bits in the bitmap.  Return
 *    -1 if 'end_pos' is less than 'begin_pos' or if either value is
 *    larger than the size of the bitmap.  Return 0 otherwise.
 */
int
skBitmapRangeClear(
    sk_bitmap_t        *bitmap,
    uint32_t            begin_pos,
    uint32_t            end_pos);


/**
 *    Return a count of the number of bits between 'begin_pos' and
 *    'end_pos' inclusive that are currently ON.  Valid values for
 *    'begin_pos' and 'end_pos' are 0 through one less than the number
 *    of bits in the bitmap.  Return UINT32_MAX if 'end_pos' is less
 *    than 'begin_pos' or if either value is larger than the size of
 *    the bitmap.
 */
uint32_t
skBitmapRangeCountHigh(
    sk_bitmap_t        *bitmap,
    uint32_t            begin_pos,
    uint32_t            end_pos);


/**
 *    Bind the bitmap iterator 'iter' to iterate over the high bits in
 *    the bitmap 'bitmap'.  It is possible to set/unset bits in the
 *    bitmap as the iterator moves through it and have the iterator
 *    visit/skip those bits---as long as the bit position changed is
 *    greater than the iterator's current position.
 */
void
skBitmapIteratorBind(
    const sk_bitmap_t  *bitmap,
    sk_bitmap_iter_t   *iter);

/**
 *    Set 'pos' to the next high bit in the bitmap that is bound to
 *    the iterator 'iter' and return SK_ITERATOR_OK.  If the iterator
 *    has visited all the high bits in the bitmap, leave the value in
 *    'pos' unchanged and return SK_ITERATOR_NO_MORE_ENTRIES.
 */
int
skBitmapIteratorNext(
    sk_bitmap_iter_t   *iter,
    uint32_t           *pos);


/**
 *    Allow the bitmap iterator 'iter' to iterate over the bitmap
 *    again.
 */
void
skBitmapIteratorReset(
    sk_bitmap_iter_t   *iter);


/**
 *  BITS_IN_WORD32(&countptr, word);
 *  BITS_IN_WORD64(&countptr, word);
 *
 *    Set the value pointed at by 'countptr' to the number of bits
 *    that are high in the value 'word', where 'word' is either 32 or
 *    64 bits.
 *
 * Fast method of computing the number of bits in a 32-bit word.  From
 * http://graphics.stanford.edu/~seander/bithacks.html#CountBitsSetParallel
 */
#define BITS_IN_WORD32(countptr, word)                                  \
    do {                                                                \
        uint32_t vv = (word) - (((word) >> 1) & 0x55555555);            \
        vv = (vv & 0x33333333) + ((vv >> 2) & 0x33333333);              \
        *(countptr) = (((vv + (vv >> 4)) & 0x0f0f0f0f) * 0x01010101) >> 24; \
    } while(0)

#define BITS_IN_WORD64(countptr, word)                                  \
    do {                                                                \
        uint64_t vv = ((uint64_t)(word)                                 \
                       - ((uint64_t)((word) >> 1)                       \
                          & UINT64_C(0x5555555555555555)));             \
        vv = ((vv & UINT64_C(0x3333333333333333))                       \
              + ((vv >> 2) & UINT64_C(0x3333333333333333)));            \
        *(countptr) = (((uint64_t)((vv + (vv >> 4))                     \
                                   & UINT64_C(0x0f0f0f0f0f0f0f0f))      \
                        * UINT64_C(0x0101010101010101)) >> 56);         \
    } while(0)

#define BITS_IN_WORD(countptr, word)    BITS_IN_WORD32(countptr, word)


/**
 *  _BITMASK32(s);
 *  _BITMASK64(s);
 *
 *    Private macro used by GET_MASKED_BITS and SET_MASKED_BITS.  This
 *    returns a 32-bit (or 64-bit) integer with the first `s'
 *    least-significant bits turned on.
 */
#define _BITMASK32(s)                                   \
    (((s) >= 32) ? UINT32_MAX : ~(UINT32_MAX << (s)))

#define _BITMASK64(s)                                   \
    (((s) >= 64) ? UINT64_MAX : ~(UINT64_MAX << (s)))

#define _BITMASK(s)  _BITMASK32(s)


/*
 *  GET_MASKED_BITS32(x, o, s)
 *  GET_MASKED_BITS64(x, o, s)
 *
 *    Given an integer value 'x', return an integer created by
 *    shifting 'x' to the right by offset 'o' bits and returning the
 *    least significant 's' bits.  Works on any size 'x' up to maximum
 *    specified in the macro's name.
 *
 *    GET_MASKED_BITS32(x, 2, 5) would return the value represented by
 *    the middle 5 bits of a single byte, as shown here:
 *
 *    76543210
 *    .xxxxx..
 *
 *    with the value shifted to the right.  The possible values
 *    returned range from 0---if NONE of the bits are set---through
 *    31---if ALL of the bits are set.
 */
#define GET_MASKED_BITS32(x, o, s) (((x) >> (o)) & _BITMASK32((s)))

#define GET_MASKED_BITS64(x, o, s) (((x) >> (o)) & _BITMASK64((s)))

#define GET_MASKED_BITS(x, o, s)   GET_MASKED_BITS32(x, o, s)


/**
 *  SET_MASKED_BITS32(x, v, o, s)
 *  SET_MASKED_BITS64(x, v, o, s)
 *
 *    Modify the integer variable 'x' by clearing the bits from 'o' to
 *    'o'+'s'-1, and replacing those bits by shifting the value 'v' to
 *    the left 'o' bits.
 *
 *    Bits are numbered with the least significant bit being 0.
 *
 *    For example, with x=0 and v<=31, the call
 *
 *    SET_MASKED_BITS32(x, v, 1, 5)
 *
 *    will result in 'x' being set to 2*'v', with a possible range of
 *    resulting values for 'x' of 2 to 62.
 *
 *    This is the setting equivalent to GET_MASKED_BITS32().
 */
#define SET_MASKED_BITS32(x, v, o, s)                   \
    do {                                                \
        (x) = (((x) & (~(_BITMASK32(s) << (o))))        \
               | (((v) & _BITMASK32(s)) << (o)));       \
    } while(0)

#define SET_MASKED_BITS64(x, v, o, s)                   \
    do {                                                \
        (x) = (((x) & (~(_BITMASK64(s) << (o))))        \
               | (((v) & _BITMASK64(s)) << (o)));       \
    } while(0)

#define SET_MASKED_BITS(x, v, o, s)  SET_MASKED_BITS32(x, v, o, s)


/**
 *  BITMAP_DECLARE(var_name, size);
 *
 *    Declares the bitmap 'var_name' that will hold size bits numbered
 *    from 0 to size-1.
 */
#define BITMAP_DECLARE(name, size)                                      \
    uint32_t name[((size) >> 5) + ((((size) & 0x1F) == 0) ? 0 : 1)]

/**
 *  BITMAP_INIT(name);
 *
 *    Clears all the bits in the bitmap named 'name'.  This macro must
 *    appear in the same scope as the BITMAP_DECLARE() macro.
 */
#define BITMAP_INIT(name) memset(&name, 0, sizeof(name))

/**
 *  BITMAP_SETBIT(name, pos);
 *  BITMAP_CLEARBIT(name, pos);
 *  is_set = BITMAP_GETBIT(name, pos);
 *
 *    Set, clear, or get the bit as position 'pos' in bitmap 'name'.
 */
#define BITMAP_SETBIT(name, pos)                        \
    ((name)[_BMAP_INDEX(pos)] |= _BMAP_OFFSET(pos))
#define BITMAP_CLEARBIT(name, pos)                      \
    ((name)[_BMAP_INDEX(pos)] &= ~_BMAP_OFFSET(pos))
#define BITMAP_GETBIT(name, pos)                                \
    (((name)[_BMAP_INDEX(pos)] & _BMAP_OFFSET(pos)) ? 1 : 0)


/*
**
**    sku-app.c
**
*/

/**
 *    The null message function.  Does nothing and returns zero.
 */
int
skMsgNone(
    const char         *fmt,
    ...)
    SK_CHECK_PRINTF(1, 2);

/**
 *    The null message function.  Does nothing and returns zero.
 */
int
skMsgNoneV(
    const char         *fmt,
    va_list             args)
    SK_CHECK_PRINTF(1, 0);

/**
 *    Register the application.  Other functions below assume you've
 *    passed argv[0] as the value of 'name'.
 */
void
skAppRegister(
    const char         *name);

/**
 *    Destroy all data structures and free all memory associated with
 *    this application.
 */
void
skAppUnregister(
    void);

/**
 *    Return the name that was used to register the application.  This
 *    is the value that was passed to skAppRegister().  The return
 *    value should be considered read-only.
 */
const char *
skAppRegisteredName(
    void);

/**
 *    Return a short name for the application.  This is the basename
 *    of the registered name.  The return value should be considered
 *    read-only.
 */
const char *
skAppName(
    void);

/**
 *    Return the full path to application.  This will consult the PATH
 *    envar and cgetcwd() to find the complete path to the
 *    application.  The return value should be considered read-only.
 */
const char *
skAppFullPathname(
    void);

/**
 *    Return the application's directory's parent directory in 'buf',
 *    a character array of 'buf_len' bytes. e.g., if the rwfilter
 *    application lives in "/usr/local/bin/rwfilter", this function
 *    puts "/usr/local" into buf.  Return value is a pointer to 'buf',
 *    or NULL on error.
 */
char *
skAppDirParentDir(
    char               *buf,
    size_t              buf_len);

/**
 *    Print short usage information---telling the user to use the
 *    ``--help'' option---to stderr and exit the application with a
 *    FAILURE exit status.
 */
void
skAppUsage(
    void)
    NORETURN;

/**
 *    Print, to the 'fh' file handle, the current application's name,
 *    the 'usage_msg', each option in 'app_options' and its 'app_help'
 *    string.  Returns control to the application.
 */
void
skAppStandardUsage(
    FILE                   *fh,
    const char             *usage_msg,
    const struct option    *app_options,
    const char            **app_help);

/**
 *    Structure used to verify that a SiLK library was compiled with
 *    the same set of features that were used to build the
 *    application.
 */
typedef struct silk_features_st {
    uint64_t struct_version;
    uint8_t  big_endian;
    uint8_t  enable_ipv6;
} silk_features_t;

#define SILK_FEATURES_DEFINE_STRUCT(var_name)   \
    const silk_features_t var_name = {          \
        2,                                      \
        SK_BIG_ENDIAN,                          \
        SK_ENABLE_IPV6                          \
    }

/**
 *    Verify that the features that were compiled into the application
 *    agree with those that were compiled into the libsilk library.
 *
 *    Currently 'future_use' is unused, but is included for possible
 *    future expansion.
 */
void
skAppVerifyFeatures(
    const silk_features_t  *features,
    void                   *future_use);

/**
 *    Sets the error stream to 'f' for any messages printed with
 *    skAppPrintErrV() or skAppUsage().  When 'f' is NULL, calling
 *    skAppPrintErr(), skAppPrintErrV(), and skAppUsage() will result
 *    in no output being generated.
 *
 *    Returns the previous value of the error-stream.
 */
FILE *
skAppSetErrStream(
    FILE               *f);

/**
 *    Sets the function that skAppPrintErr() will call to print its
 *    arguments.  If 'fn' is NULL, resets skAppPrintErr() to use its
 *    default function, which is skAppPrintErrV().  To disable
 *    printing of errors, pass 'skMsgNoneV' to this function.
 */
void
skAppSetFuncPrintErr(
    sk_msg_vargs_fn_t   fn);

/**
 *    Sets the function that skAppPrintSyserror() will call to print
 *    its arguments.  If 'fn' is NULL, resets skAppPrintSyserror() to
 *    use its default function, which is skAppPrintSyserrorV().  To
 *    disable printing of errors, pass 'skMsgNoneV' to this function.
 */
void
skAppSetFuncPrintSyserror(
    sk_msg_vargs_fn_t   fn);

/**
 *    Sets the function that skAppPrintAbortMsg(),
 *    skAppPrintBadCaseMsg(), and skAppPrintOutOfMemoryMsgFunction()
 *    will call to print their arguments.  If 'fn' is NULL, resets the
 *    functions to use the default function, which is skAppPrintErr().
 *    To disable printing of errors, pass 'skMsgNone' to this
 *    function.
 */
void
skAppSetFuncPrintFatalErr(
    sk_msg_fn_t         fn);

/**
 *    Prints the application name, a colon, the result of formatting
 *    the arguments using v*printf(), and a newline to the stream set
 *    by skAppSetErrStream(), which defaults to stderr.
 *
 *    This is the default function used by skAppPrintErr() to print
 *    error messages.
 */
int
skAppPrintErrV(
    const char         *fmt,
    va_list             args);

/**
 *    Prints the application name, a colon, the result of formatting
 *    the arguments using v*printf(), the result of calling
 *    strerror(), and a newline to the stream set by
 *    skAppSetErrStream(), which defaults to stderr.
 *
 *    This is the default function used by skAppPrintSyserror() to
 *    print system error messages.
 */
int
skAppPrintSyserrorV(
    const char         *fmt,
    va_list             args);


#ifdef TEST_PRINTF_FORMATS
#define skAppPrintErr printf
#else
/**
 *    Calls the function set by skAppSetFuncPrintErr()---which has the
 *    default value skAppPrintErrV()---to format and print the
 *    arguments using the given format 'fmt'.
 */
int
skAppPrintErr(
    const char         *fmt,
    ...)
    SK_CHECK_PRINTF(1, 2);
#endif


#ifdef TEST_PRINTF_FORMATS
#define skAppPrintSyserror printf
#else
/**
 *    Calls the function set by skAppSetFuncPrintSyserror()---which
 *    has the default value skAppPrintSyserrorV()---to format and
 *    print the arguments using the given format 'fmt'.
 */
int
skAppPrintSyserror(
    const char         *fmt,
    ...)
    SK_CHECK_PRINTF(1, 2);
#endif


/**
 *    Prints a message to the standard error.  This function appends a
 *    final newline to the output.  This function may used by
 *    TRACEMSG(), see sktracemsg.h.
 */
int
skTraceMsg(
    const char         *fmt,
    ...)
    SK_CHECK_PRINTF(1, 2);


/**
 *    Install the function 'sig_handler' to be called whenever the
 *    application receives a SIGINT, SIGTERM, SIGQUIT, or SIGPIPE.
 *
 *    The parameter passed to 'sig_handler' will be the received
 *    signal.
 *
 *    Return 0 on success, or -1 if the handler cannot be installed.
 */
int
skAppSetSignalHandler(
    void              (*sig_handler)(int signal));


/**
 *    Use skAbort() to call this function.
 *
 *    skAbort() is a macro defined in silk.h that calls this function
 *    with the current function name (if the compiler supports it),
 *    filename, and line number macros.  Once this function returns,
 *    the skAbort() macro calls abort().
 *
 *    This function will call the function specified by
 *    skAppSetFuncPrintFatalErr() to print a message containing the
 *    specified 'func_name', 'file_name' and 'line_number' values.
 *    This function will return.  It is the caller's responsibility to
 *    call abort().
 */
void
skAppPrintAbortMsg(
    const char         *func_name,
    const char         *file_name,
    int                 line_number);


/**
 *    Use skAbortBadCase(case_expr) to call this function.
 *
 *    skAbortBadCase() is a macro defined in silk.h that calls this
 *    function with the current function name (if the compiler
 *    supports it), filename, line number macros, the value of
 *    'case_expr' as an int64_t and the stringification of
 *    'case_expr'.  Once this function returns, the skAbortBadCase()
 *    macro calls abort().
 *
 *    This function will call the function specified by
 *    skAppSetFuncPrintFatalErr() to print an appropriate message
 *    regarding switch(value_expr) getting an unexpected value.  This
 *    function will return.  It is the caller's responsibility to call
 *    abort().
 */
void
skAppPrintBadCaseMsg(
    const char         *func_name,
    const char         *file_name,
    int                 line_number,
    int64_t             value,
    const char         *value_expr);


#ifdef SK_HAVE_C99___FUNC__
#define skAppPrintOutOfMemory(oom_string)                                  \
    skAppPrintOutOfMemoryMsgFunction(__func__, __FILE__, __LINE__, oom_string)
#else
#define skAppPrintOutOfMemory(oom_string)                                  \
    skAppPrintOutOfMemoryMsgFunction(NULL, __FILE__, __LINE__, oom_string)
#endif


/**
 *    Use skAppPrintOutOfMemory("object_name") to call this function.
 *
 *    skAppPrintOutOfMemory() is a macro defined above that calls this
 *    function with the current function name (if the compiler
 *    supports it), filename, line number macros, the value for
 *    'object_name'.
 *
 *    This function will call the function specified by
 *    skAppSetFuncPrintFatalErr() to print a message about being out
 *    of memory and unable to allocate 'object_name'.  'object_name'
 *    may be NULL.  This function will return.  It is the caller's
 *    responsibility to exit the program.
 */
void
skAppPrintOutOfMemoryMsgFunction(
    const char         *func_name,
    const char         *file_name,
    int                 line_number,
    const char         *object_name);


/*
**
**    sku-options.c
**
*/

/*
 *    Values to specify for the 'has_arg' member of struct option when
 *    calling skOptionsRegister().
 */
#define NO_ARG          0
#define REQUIRED_ARG    1
#define OPTIONAL_ARG    2


/**
 *    Macro that converts the preceeding values to a string; used when
 *    printing usage (--help) information.  The paraemter to the macro
 *    is a struct option.
 */
#define SK_OPTION_HAS_ARG(struct_option_st)             \
    (((struct_option_st).has_arg == REQUIRED_ARG)       \
     ? "Req Arg"                                        \
     : (((struct_option_st).has_arg == OPTIONAL_ARG)    \
        ? "Opt Arg"                                     \
        : (((struct_option_st).has_arg == NO_ARG)       \
           ? "No Arg"                                   \
           : "BAD 'has_arg' VALUE")))

/**
 *    Callback data type for the options handler function.
 */
typedef void   *clientData;

/**
 *    Signature of the options handler callback function to pass to
 *    skOptionsRegister().  An options handler function is invoked by
 *    skOptionsParse() for each command line switch.
 *
 *    'cData' is user callback data that was supplied to
 *    'skOptionsRegister().  The 'optIndex' parameter is set to the
 *    value of the 'val' member of the struct option structure for the
 *    option.  'optArg' is the user's argument to the command line
 *    switch, or NULL if the user provided no switch.
 *
 *    The options handler should return 0 on success, or non-zero if
 *    there was a problem processing the option.  A non-zero return
 *    value causes options parsing to stop.
 */
typedef int (*optHandler)(
    clientData  cData,
    int         optIndex,
    char       *optArg);

/**
 *    Signature of callback functions invoked when the --help or
 *    --version switches are seen.
 */
typedef void  (*usage_fn_t)(
    void);

/**
 *    Register command line switches (options) with the global options
 *    handler.  The paramter 'options' specifies the switches to
 *    register as an array of:
 *
 *        struct option {
 *            char   *name;
 *            int     has_arg;
 *            int    *flag;
 *            int     val;
 *        };
 *
 *    where 'name' is the switch name (without the preceeding
 *    hyphens), 'has_arg' is one of NO_ARG, REQUIRED_ARG, OPTIONAL_ARG
 *    depending on whether the switch requires an argument, 'flag'
 *    should always be 0, and 'val' is a user-supplied value that gets
 *    passed as the 'optIndex' parameter of the options handler
 *    callback.
 *
 *    Processing of the 'options' array stops once a NULL value for
 *    'name' is found or once 'max_options' elements of the 'options'
 *    array have been processed, whichever occurs first.  A
 *    'max_options' value of 0 is equivalent to no limit.
 *
 *    The 'handler' function is the callback function to invoke when
 *    the option is seen on the command line.
 *
 *    The 'cData' parameter is a caller maintained pointer will be
 *    supplied to the handler function.  skOptionsRegisterCount() does
 *    not use this value.  It may be NULL.
 */
int
skOptionsRegisterCount(
    const struct option    *options,
    size_t                  max_options,
    optHandler              handler,
    clientData              cData);

/**
 *    A wrapper for skOptionsRegisterCount() that calls that function
 *    with 0 for the 'max_options' parameter.
 */
int
skOptionsRegister(
    const struct option    *options,
    optHandler              handler,
    clientData              cData);

/**
 *    Prepare the application to process options.
 *
 *    Application writers do not need to call this function as it is
 *    called by skAppRegister().
 */
void
skOptionsSetup(
    void);

/**
 *    Free all memory associated with options processing.
 *
 *    Application writers do not need to call this function as it is
 *    called by skAppUnregister().
 */
void
skOptionsTeardown(
    void);

/**
 *    Register the function 'help_fn' as the function the appliction
 *    will invoke when the user runs the application with the --help
 *    option.  If not specified, no output is produced by --help.
 *
 *    This function should print its results to the stdout.
 *
 *    It is recommended that this function print a brief summary of
 *    the purpose of the application, and then print every switch the
 *    function accepts.  The skAppStandardUsage() function can be used
 *    to print much of this information.
 */
void
skOptionsSetUsageCallback(
    usage_fn_t          help_fn);

/**
 *    Register the function 'version_fn' as the function the
 *    application will invoke when the user runs the application with
 *    the --version option.  If not specified, information about the
 *    version of libsilk that the application is using will be
 *    printed.
 *
 *    This function should print its results to the stdout.
 *
 *    It is recommended that this function print information about how
 *    the application was compiled and the license(s) that the
 *    application is released under.
 */
void
skOptionsSetVersionCallback(
    usage_fn_t          version_fn);

/**
 *    Print usage information about the default options that all
 *    applications support (namely --help and --version) to the named
 *    file handle.
 */
void
skOptionsDefaultUsage(
    FILE               *fh);


#ifndef SK_SUPPORT_CONF_FILE
#  define SK_SUPPORT_CONF_FILE 0
#endif
#if  SK_SUPPORT_CONF_FILE
/**
 * optionsHandleConfFile:
 *
 *     Loads a configuration file.  The configuration file consists of
 *     a series of newline-terminated lines.  A line consisting of
 *     only whitespace, or whose first non-whitespace character is a
 *     `#' character is ignored.  All other lines should consist of a
 *     single option name followed by the option's value (if any),
 *     separated by whitespace.  Whitespace at the beginning and end
 *     of the line is ignored.
 *
 * BUGS:
 *     If you intersperse switches (options) and arguments, arguments
 *     before the configuration file is parsed will not be seen.
 *
 *  Return:
 *      0 if ok. -1 else
 */
int
optionsHandleConfFile(
    char               *filename);
#endif  /* SK_SUPPORT_CONF_FILE */


/**
 *  skOptionsParse:
 *      Adjust the global options array to allow for the help
 *      option. If help is selected by the user, call the stashed
 *      usageFunction.  Parse input options given a set of
 *      pre-registered options and their handlers.  For each
 *      legitimate option, call the handler.
 *  SideEffects:
 *      The individual handlers update whatever datastruture they wish
 *      to via the clientData argument to the handler.
 *  Return:
 *      optind which points at the first non-option argument passed if
 *      all is OK.  If not OK, the return -1 for error.
 */
int
skOptionsParse(
    int                 argc,
    char              **argv);


/**
 *    Verify that the directory in 'dirname' exists, that the length
 *    is shorter than PATH_MAX, and that we have a full path to the
 *    directory.  If so, return 0; otherwise, print an error that the
 *    option named by 'option_name' was bad and return -1.
 */
int
skOptionsCheckDirectory(
    const char         *dirname,
    const char         *option_name);


/**
 *    Return the length of the shortest unique prefix required to
 *    match the option whose complete name is 'option_name'.  Return
 *    -1 if option_name is NULL, empty, or if no options have the name
 *    'option_name'.
 */
int
skOptionsGetShortestPrefix(
    const char         *option_name);


/**
 *    Registers a --temp-directory switch for the application.  Use
 *    skOptionsTempDirUsage() to print the usage for this switch.
 *
 *    The parameter 'var_location' must be specified.  The variable at
 *    that location will be set to the location the user provides in
 *    the --temp-directory switch.
 *
 *    The variable at 'var_location' is only modified if the user
 *    specifies the --temp-directory switch,
 */
int
skOptionsTempDirRegister(
    const char        **var_location);

/**
 *    Print the usage information for the --temp-directory switch.
 */
void
skOptionsTempDirUsage(
    FILE               *fh);


/*
**
**    skqsort.c
**
*/

/**
 *    Perform a quicksort on the elements in 'buffer', where each
 *    element is 'element_size' octets and 'buffer' contains
 *    'number_elements' such elements.
 *
 *    The elements in 'buffer' will be sorted in assending order
 *    according to the comparison function 'cmp'.  'cmp' will be
 *    handed pointers to two elements in the buffer, and it must
 *    return a value less than, equal to, or greater than zero if the
 *    first element is less than, equal to, or greater than the
 *    second.
 */
void
skQSort(
    void               *buffer,
    size_t              number_elements,
    size_t              element_size,
    int               (*cmp)(const void *a, const void *b));

/**
 *    Perform the same quicksort operation as skQSort().  This
 *    function takes a context object 'thunk' which the function
 *    passes as the third argument to the comparison function 'cmp'.
 */
void
skQSort_r(
    void               *buffer,
    size_t              number_elements,
    size_t              element_size,
    int               (*cmp)(const void *a, const void *b, void *thunk),
    void               *thunk);


/**
 *    Registers an --ip-format switch for the application that allows
 *    the user to specify how IP addresses will be displayed.
 *
 *    The parameter 'var_location' must be specified.  The variable at
 *    that location will be set to the value the user specifies in the
 *    various switches.  This value will be one of the values defined
 *    in skipaddr_flags_t (see silk_types.h).
 *
 *    Use skOptionsIPFormatUsage() to print the usage for these
 *    switches.
 *
 *    The 'settings' parameter registers additional switches.
 *
 *    When SK_OPTION_IP_FORMAT_INTEGER_IPS is specified, an
 *    --integer-ips switch is also registered.
 *
 *    When SK_OPTION_IP_FORMAT_ZERO_PAD_IPS is specified, a
 *    --zero-pad-ips switch is also registered.
 *
 *    When SK_OPTION_IP_FORMAT_UNMAP_V6 is specified, unmap-v6
 *    (SKIPADDR_UNMAP_V6) is automatically added to the default IP
 *    format and to the result of parsing the --ip-format switch
 *    unless that format specifies map-v4 explicitly or the output
 *    format is decimal or hexadecimal.
 *
 *    The variable at 'var_location' is only modified if the user
 *    specifies the --ip-format switch.
 *
 *    If the SILK_IP_FORMAT environment variable is set, the value in
 *    'var_location' is initialized with the result of parsing that
 *    variable's value as if it was an argument to the --ip-format
 *    switch.
 *
 *    The 'settings' parameter was added in SiLK 3.15.0.
 */
int
skOptionsIPFormatRegister(
    uint32_t           *var_location,
    uint32_t            settings);

/**
 *    Print the usage information for the switches registered by
 *    skOptionsIPFormatRegister() to the stream 'fp'.
 */
void
skOptionsIPFormatUsage(
    FILE               *fp);

/**
 *    A bit to include in the 'settings' argument to
 *    skOptionsIPFormatRegister() that indicates an --integer-ips
 *    switch should be included.  Since SiLK 3.15.0.
 */
#define SK_OPTION_IP_FORMAT_INTEGER_IPS     (1u << 0)

/**
 *    A bit to include in the 'settings' argument to
 *    skOptionsIPFormatRegister() that indicates an --zero-pad-ips
 *    switch should be included.  Since SiLK 3.15.0.
 */
#define SK_OPTION_IP_FORMAT_ZERO_PAD_IPS    (1u << 1)

/**
 *    A bit to include in the 'settings' argument to
 *    skOptionsIPFormatRegister() that causes SKIPADDR_UNMAP_V6
 *    (unmap-v6) to be automatically added to the default IP format
 *    and to the result of parsing the --ip-format switch unless that
 *    format specifies map-v4 explicitly or the output format is
 *    decimal or hexadecimal.  Since SiLK 3.17.0.
 */
#define SK_OPTION_IP_FORMAT_UNMAP_V6        (1u << 2)


/*
**
**    skoptions-notes.c
**
*/


int
skOptionsNotesRegister(
    int                *note_strip);

void
skOptionsNotesTeardown(
    void);

void
skOptionsNotesUsage(
    FILE               *fh);

int
skOptionsNotesAddToStream(
    skstream_t         *stream);



/*
**
**    sku-filesys.c
**
*/

/**
 *    Strip directory prefix from the file path fp.  Returns a pointer
 *    to a static string buffer.
 */
char *
skBasename(
    const char         *fp);


/**
 *    Thread safe version of skBasename()
 */
char *
skBasename_r(
    char               *dest,
    const char         *src,
    size_t              dest_size);


/**
 *    Strip file name suffix from the file path fp.  Returns a pointer
 *    to a static string buffer.
 */
char *
skDirname(
    const char         *fp);


/**
 *    Thread safe version of skDirname()
 */
char *
skDirname_r(
    char               *dest,
    const char         *src,
    size_t              dest_size);


/**
 *    Returns 1 if the FILE* fd is a tty, 0 otherwise
 */
#define FILEIsATty(fd)          isatty(fileno(fd))


/**
 *    Return 1 if 'path_name' exists and is a FIFO.
 *
 *    Return 0 and set errno if calling stat() on 'path_name' fails.
 *
 *    Return 0 when stat() succeeds but 'path_name' is not a FIFO.
 *
 *    See also skFileExists().
 */
int
isFIFO(
    const char         *path_name);


/**
 *    Return 1 if 'path_name' exists and is a directory.
 *
 *    Return 0 and set errno if calling stat() on 'path_name' fails.
 *
 *    Return 0 when stat() succeeds but 'path_name' is not a
 *    directory.
 */
int
skDirExists(
    const char         *path_name);


/**
 *    Return 1 if 'path_name' exists and is either a regular file or a
 *    FIFO.
 *
 *    Return 0 and set errno if calling stat() on 'path_name' fails.
 *
 *    Return 0 when stat() succeeds but 'path_name' is neither a
 *    regular file nor a FIFO.
 *
 *    See also skDirExists() and isFIFO().
 */
int
skFileExists(
    const char         *path_name);


/**
 *    Return the size of 'path_name' if it exists.
 *
 *    Return 0 if 'path_name' exists and is an empty file.
 *
 *    Return 0 and set errno if calling stat() on 'path_name' fails.
 *
 *    See also skFileExists().
 */
off_t
skFileSize(
    const char         *path_name);


/**
 *    Perform a locking operation on the opened file represented by
 *    the file descriptor 'fd'.  'type' is the type of lock, it should
 *    be one of F_RDLCK for a read lock, F_WRLCK for a write lock, or
 *    F_UNLCK to unlock a previously locked file.  'cmd' should be one
 *    of F_SETLKW to wait indefinitely for a lock, or F_SETLK to
 *    return immediately.  Return 0 if successful, -1 otherwise.  Use
 *    errno to determine the error.
 */
int
skFileSetLock(
    int                 fd,
    short               type,
    int                 cmd);


/**
 *    Make the complete directory path to 'directory', including
 *    parent(s) if required.  A return status of 0 indicates that
 *    'directory' exists---either this function created it or it
 *    existed prior to invoking this function.
 *
 *    A return status of 1 indicates a failure to create 'directory'
 *    or its parent(s), or that 'directory' already exists and it is
 *    not a directory.  On failure, any parent directories that were
 *    created by this function will not be removed.  This function
 *    sets errno on failure.
 */
int
skMakeDir(
    const char         *directory);


/**
 *    Copy the file 'src_path' to 'dest_path'.  'dest_path' may be a
 *    file or a directory.  Overwrite the destination file if it
 *    already exists.  Return 0 on success, or errno on failure.
 */
int
skCopyFile(
    const char         *src_path,
    const char         *dest_path);


/**
 *    Move the file at 'src_path' to 'dest_path'.  'dest_path' may be
 *    a file or a directory.  Overwrite the destination file if it
 *    already exists.  Return 0 on success, or errno on failure.
 */
int
skMoveFile(
    const char         *src_path,
    const char         *dest_path);


/**
 *    Return the location of the temporary directory.
 *
 *    If a temporary directory is not specified or if the specified
 *    location does not exist, return NULL.  In addition, if 'err_fn'
 *    is specified, print an error message using that function.
 *
 *    If 'user_temp_dir' is specified, that location is used.
 *    Otherwise, locations specified by the SILK_TMPDIR and TMPDIR
 *    environment variables are checked, in that order.  Finally, the
 *    DEFAULT_TEMP_DIR is used if defined.
 */
const char *
skTempDir(
    const char         *user_temp_dir,
    sk_msg_fn_t         err_fn);


/**
 *    DEPRECATED.  Use skFileptrOpenPager() instead.
 *
 *    Attempts to redirect the '*output_stream' to the paging program
 *    '*pager.'
 *
 *    If output changed so that it goes to a pager, 1 is returned; if
 *    the output is unchanged, 0 is returned.  If an error occurred in
 *    invoking the pager, -1 is returned.
 *
 *    If the '*output_stream' is NULL, it is assumed that the output
 *    was being sent to stdout.  If the '*output_stream' is not stdout
 *    or not a terminal, no paging is preformed and 0 is returned.
 *
 *    If '*pager' is NULL, the environment variable SILK_PAGER is
 *    checked for the paging program; if that is NULL, the PAGER
 *    environment variable is checked.  If that is also NULL, no
 *    paging is performed and 0 is returned.
 *
 *    If there was a problem invoking the pager, an error is printed
 *    and -1 is returned.
 *
 *    If the pager was started, the *output_stream value is set to the
 *    pager stream, and *pager is set to the name of the pager (if the
 *    user's environment was consulted), and 1 is returned.  To close
 *    the pager, use the skClosePager() function, or call pclose() on
 *    the *output_stream.
 *
 *    Due to the race condition of checking the status of the child
 *    process, it is possible for 1 to be returned even though the
 *    pager has exited.  In this case the tool's output will be lost.
 */
int
skOpenPagerWhenStdoutTty(
    FILE              **output_stream,
    char              **pager);


/**
 *    DEPRECATED
 *
 *    If skOpenPagerWhenStdoutTty() returns a positive value, use this
 *    function to close the pager stream.  Prints an error if the
 *    close fails.
 */
void
skClosePager(
    FILE               *output_stream,
    const char         *pager);


/**
 *    Fill 'out_buffer' with the next non-blank, non-comment line read
 *    from 'stream'.  The caller should supply 'out_buffer' and pass
 *    its size in the 'buf_size' variable.  The return value is the
 *    number of lines that were read to get a valid line.
 *
 *    The final newline will be removed; if comment_start is provided,
 *    it will also be removed from line.
 *
 *    If a line longer than buf_size is found, out_buffer will be set
 *    to the empty string but the return value will be positive.
 *
 *    At end of file, 0 is returned and out_buffer is the empty
 *    string.
 */
int
skGetLine(
    char               *out_buffer,
    size_t              buf_size,
    FILE               *stream,
    const char         *comment_start);


/**
 *  Reads 'count' bytes from the file descriptor 'fd' into the buffer
 *  'buf'.  The return value is the number of bytes actually read.
 *  Will return less than 'count' if an end-of-file situation is
 *  reached.  In an error condition, returns -1.  More information
 *  about the error can be derived from 'errno'.
 */
ssize_t
skreadn(
    int                 fd,
    void               *buf,
    size_t              count);

/**
 *  Writes 'count' bytes from the buffer 'buf' to the file descriptor
 *  'fd'.  The return value is the number of bytes actually written.
 *  If the underlying `write' call returns zero, this function will
 *  return a value less than 'count'.  It is unknown if this actually
 *  occurs in any real-life conditions.  In an error condition,
 *  returns -1.  More information about the error can be derived from
 *  'errno'.
 */
ssize_t
skwriten(
    int                 fd,
    const void         *buf,
    size_t              count);



/*
**
**    sku-ips.c
**
*/


/**
 *    Return the log2 of 'value' as an integer.  This is the position
 *    of the most significant bit in 'value', assuming the MSB is
 *    number 63 and the LSB is 0.  Returns -1 if 'value' is 0.
 */
int
skIntegerLog2(
    uint64_t            value);


/**
 *    Compute the largest CIDR block that begins at 'start_addr' and
 *    contains no IPs larger than 'end_addr', and return the CIDR
 *    designation for that block.  For example:
 *
 *        skCIDRComputePrefix(10.0.0.2, 10.0.0.5, NULL) => 31
 *
 *    When 'new_start_addr' is not NULL, it's value is set to 0 if the
 *    CIDR block completely contains all IPs between 'start_addr' and
 *    'end_addr' inclusive.  Otherwise, its value is set to the IP that
 *    follows that block covered by the CIDR block.
 *
 *    Returns -1 if 'end_addr' < 'start_addr'.
 *
 *    This function allows one to print all CIDR blocks from 'start'
 *    to 'end' using:
 *
 *        do {
 *            cidr_prefix = skCIDRComputePrefix(&start, &end, &new_start);
 *            printf("%s/%d", skipaddrString(buf, start, 0), cidr_prefix);
 *            skipaddrCopy(&start, &new_start);
 *        } while (!skipaddrIsZero(&start));
 *
 *    Continuing the above example:
 *
 *        skCIDRComputePrefix(10.0.0.2, 10.0.0.5, &new) => 31, new => 10.0.0.4
 *        skCIDRComputePrefix(10.0.0.4, 10.0.0.5, &new) => 31, new => 0
 *
 *    which means that the IP range 10.0.0.2--10.0.0.5 is contained by
 *    the two CIDR blocks:
 *
 *        10.0.0.2/31
 *        10.0.0.4/31
 */
int
skCIDRComputePrefix(
    const skipaddr_t   *start_addr,
    const skipaddr_t   *end_addr,
    skipaddr_t         *new_start_addr);

/**
 *    Older interface to skCIDRComputePrefix() that uses integers for
 *    the IP addresses.
 */
int
skComputeCIDR(
    uint32_t            start_ip,
    uint32_t            end_ip,
    uint32_t           *new_start_ip);


/**
 *    Compute the minimum and maximum IPs that are represented by
 *    'ipaddr' when the CIDR mask 'cidr' is applied to it.  Return -1
 *    if 'cidr' is too large for the type of IP address.
 *
 *    Either of the output values, 'min_ip' or 'max_ip', may point to
 *    the same memory as 'ipaddr'.
 *
 *    See also skCIDRComputeEnd().
 */
int
skCIDR2IPRange(
    const skipaddr_t   *ipaddr,
    uint32_t            cidr,
    skipaddr_t         *min_ip,
    skipaddr_t         *max_ip);


/**
 *    Compute the minimum IP that is covered by the CIDR block
 *    'ipaddr'/'cidr' and fill 'min_ip' with that value.  Return -1 if
 *    'cidr' is too marge for the type of IP address.
 *
 *    'min_ip' may point to the same memory as 'ipaddr'.
 *
 *    See also skCIDR2IPRange().
 */
int
skCIDRComputeStart(
    const skipaddr_t   *ipaddr,
    uint32_t            cidr,
    skipaddr_t         *min_ip);


/**
 *    Compute the maximum IP that is covered by the CIDR block
 *    'ipaddr'/'cidr' and fill 'max_ip' with that value.  Return -1 if
 *    'cidr' is too marge for the type of IP address.
 *
 *    'max_ip' may point to the same memory as 'ipaddr'.
 *
 *    See also skCIDR2IPRange().
 */
int
skCIDRComputeEnd(
    const skipaddr_t   *ipaddr,
    uint32_t            cidr,
    skipaddr_t         *max_ip);


/**
 *    Parse the string 'policy_name' and set 'ipv6_policy' to the
 *    parsed value.  Return 0 on success.
 *
 *    If 'policy_name' is not a valid policy, return -1.  In addition,
 *    if 'option_name' is non-NULL, print an error message that the
 *    'policy_name' was invalid.
 */
int
skIPv6PolicyParse(
    sk_ipv6policy_t    *ipv6_policy,
    const char         *policy_name,
    const char         *option_name);


/**
 *    Add an option that will allow the user to determine how IPv6
 *    flow records are handled.  This function will also check the
 *    environment variabled namd by SILK_IPV6_POLICY_ENVAR for a
 *    policy to follow.  After skOptionsParse() sucessfully returns, the
 *    variable pointed to by 'ipv6_policy' will contain the IPv6
 *    policy to follow.
 *
 *    Before calling this function, the caller should set the variable
 *    that ipv6_policy points at to the application's default IPv6
 *    policy.
 *
 *    If IPv6 support is not enabled, the ipv6_policy is set to ignore
 *    IPv6 flows, the environment is not checked, and no option is
 *    registered
 */
int
skIPv6PolicyOptionsRegister(
    sk_ipv6policy_t    *ipv6_policy);


/**
 *    Print the help text for the --ipv6-policy switch to the named
 *    file handle.  Uses the default policy that was set when
 *    skIPv6PolicyOptionsRegister() was called.
 */
void
skIPv6PolicyUsage(
    FILE               *fh);


/* typedef struct skIPWildcard_st skIPWildcard_t; // silk_types.h */
struct skIPWildcard_st {
    /*
     *    m_blocks[] contains a bitmap for each octet of an IPv4
     *    address (IPv6:hexadectet).  If the bit is enabled, that
     *    value should be returned for that octet/hexadectet.
     *
     *    m_min[] and m_max[] are the minimum and maximum values in
     *    the bitmap for the octet/hexadectet.
     *
     *    num_blocks is 4 for an IPv4 address, 8 for an IPv6 address.
     *
     *    m_blocks[0], m_min[0], m_max[0] represent the values for the
     *    most significant octet/hexadectet.
     */
#if !SK_ENABLE_IPV6
    uint32_t            m_blocks[4][256/32];
    uint16_t            m_min[4];
    uint16_t            m_max[4];
#else
    uint32_t            m_blocks[8][65536/32];
    uint16_t            m_min[8];
    uint16_t            m_max[8];
#endif
    uint8_t             num_blocks;
};

typedef struct skIPWildcardIterator_st {
    const skIPWildcard_t   *ipwild;
    uint16_t                i_block[8];
    unsigned                no_more_entries :1;
    unsigned                force_ipv6      :1;
    unsigned                force_ipv4      :1;
} skIPWildcardIterator_t;


#define _IPWILD_BLOCK_IS_SET(ipwild, block, val)                        \
    ((ipwild)->m_blocks[(block)][_BMAP_INDEX(val)] & _BMAP_OFFSET(val))

#define _IPWILD_IPv4_IS_SET(ipwild, ipaddr)                             \
    (_IPWILD_BLOCK_IS_SET((ipwild), 0, 0xFF&(skipaddrGetV4(ipaddr) >> 24))&& \
     _IPWILD_BLOCK_IS_SET((ipwild), 1, 0xFF&(skipaddrGetV4(ipaddr) >> 16))&& \
     _IPWILD_BLOCK_IS_SET((ipwild), 2, 0xFF&(skipaddrGetV4(ipaddr) >>  8))&& \
     _IPWILD_BLOCK_IS_SET((ipwild), 3, 0xFF&(skipaddrGetV4(ipaddr))))


/**
 *    Zero all values in the skIPWildcard_t 'ipwild'.
 */
void
skIPWildcardClear(
    skIPWildcard_t     *ipwild);



#if !SK_ENABLE_IPV6
#  define skIPWildcardIsV6(ipwild)   0
#  define skIPWildcardCheckIp(ipwild, ipaddr) \
    _IPWILD_IPv4_IS_SET(ipwild, ipaddr)
#else
#  define skIPWildcardIsV6(ipwild) (8 == (ipwild)->num_blocks)
/**
 *    Return 1 if 'ip' is represented in by 'ipwild'; 0 otherwise.
 */
int
skIPWildcardCheckIp(
    const skIPWildcard_t   *ipwild,
    const skipaddr_t       *ip);
#endif


/**
 *    Bind the iterator 'iter' to iterate over the entries in the
 *    skIPWildcard_t 'ipwild'.  Use skIPWildcardIteratorNext() or
 *    skIPWildcardIteratorNextCidr() to get the entries.
 *
 *    When iterating over the IPs, the addresses will be returned in
 *    the form in which they were specified when 'ipwild' was
 *    created---that is, as either IPv4 or as IPv6.  To force an
 *    IPWildcard of IPv4 addresses to be returned as IPv6 addresses
 *    (in the ::ffff:0:0/96 subnet), use skIPWildcardIteratorBindV6().
 *
 *    Return 0 unless 'ipwild' is NULL.
 */
int
skIPWildcardIteratorBind(
    skIPWildcardIterator_t *iter,
    const skIPWildcard_t   *ipwild);


#if SK_ENABLE_IPV6
/**
 *    Bind the iterator 'iter' to iterate over the entries in the
 *    skIPWildcard_t 'ipwild'.  Similar to skIPWildcardIteratorBind(),
 *    but instructs the iterator to return IPv6 addresses even when
 *    the 'ipwild' contains IPv4 addresses.  The IPv4 addresses are
 *    mapped into the ::ffff:0:0/96 subnet.
 */
int
skIPWildcardIteratorBindV6(
    skIPWildcardIterator_t *iter,
    const skIPWildcard_t   *ipwild);

/**
 *    Bind the iterator 'iter' to iterate over the entries in the
 *    skIPWildcard_t 'ipwild'.  Similar to skIPWildcardIteratorBind(),
 *    but instructs the iterator to only visit IPv6 addresses within
 *    the ::ffff:0:0/96 subnet and return them as IPv4 addresses.
 *
 *    Since SiLK 3.9.0.
 */
int
skIPWildcardIteratorBindV4(
    skIPWildcardIterator_t *out_iter,
    const skIPWildcard_t   *ipwild);
#else  /* SK_ENABLE_IPV6 */
#define skIPWildcardIteratorBindV4 skIPWildcardIteratorBind
#endif  /* SK_ENABLE_IPV6 */


/**
 *    Fill 'out_ip' with the next IP address represented by the
 *    IPWildcard that is bound to the iterator 'iter'.  Return
 *    SK_ITERATOR_OK if 'out_ip' was filled with an IP address, or
 *    SK_ITERATOR_NO_MORE_ENTRIES otherwise.
 *
 *    This function can be intermingled with calls to
 *    skIPWildcardIteratorNextCidr(), which returns the CIDR blocks in
 *    the IPWildcard.
 */
skIteratorStatus_t
skIPWildcardIteratorNext(
    skIPWildcardIterator_t *iter,
    skipaddr_t             *out_ip);


/**
 *    Fill 'out_min_ip' and 'out_prefix' with the next CIDR block
 *    (subnet) represented by the IPWildcard that is bound to the
 *    iterator 'iter'.  Return SK_ITERATOR_OK if 'out_min_ip' was
 *    filled with an IP address, or SK_ITERATOR_NO_MORE_ENTRIES
 *    otherwise.
 *
 *    This function can be intermingled with calls to
 *    skIPWildcardIteratorNext(), which returns the individual IPs in
 *    the IPWildcard.
 */
skIteratorStatus_t
skIPWildcardIteratorNextCidr(
    skIPWildcardIterator_t *iter,
    skipaddr_t             *out_min_ip,
    uint32_t               *out_prefix);


/**
 *    Allow 'iter' to iterate again over the entries in the IPWildcard
 *    that is bound to it.
 */
void
skIPWildcardIteratorReset(
    skIPWildcardIterator_t *iter);


/*
**
**    sku-string.c
**
*/

/**
 *    Fill 'outbuf' with a string representation of the IP address in
 *    'ip'.  The size of 'outbuf' must be at least SKIPADDR_STRLEN bytes
 *    in length.  The function returns a pointer to 'outbuf'.
 *
 *    The form of the string will depend on the values in 'ip_flags',
 *    which should contain values from the skipaddr_flags_t
 *    enumeration.  Both skipaddr_flags_t and SKIPADDR_STRLEN are
 *    defined in silk_types.h.
 *
 *    The strlen() of the returned string depends on the values in
 *    'ip_flags'.  For help in setting the column width of the output,
 *    use skipaddrStringMaxlen() to get the length of the longest
 *    possible string.
 *
 *    Use skipaddrCidrString() for when a CIDR prefix (netblock)
 *    length follows the IP address.
 */
char *
skipaddrString(
    char               *outbuf,
    const skipaddr_t   *ip,
    uint32_t            ip_flags);

/**
 *    Fill 'outbuf' with a string representation of the CIDR block
 *    (netblock) denoted by the IP address in 'ip' and length in
 *    'prefix'.  The size of 'outbuf' must be at least
 *    SKIPADDR_CIDR_STRLEN bytes in length.  The function returns a
 *    pointer to 'outbuf'.
 *
 *    If the 'ip_flags' changes the displayed IP address between IPv4
 *    and IPv6 then the displayed 'prefix' is adjusted as well.
 *
 *    Note: The function assumes the proper mask has already been
 *    applied to the 'ip' (that is, that the bits below the 'prefix'
 *    are zero).
 *
 *    The form of the string will depend on the values in 'ip_flags',
 *    which should contain values from the skipaddr_flags_t
 *    enumeration.  Both skipaddr_flags_t and SKIPADDR_CIDR_STRLEN are
 *    defined in silk_types.h.
 *
 *    When 'ip_flags' contains SKIPADDR_ZEROPAD, the CIDR prefix value
 *    is also zero-padded.  The SKIPADDR_HEXADECIMAL setting does does
 *    affect the CIDR prefix, which is always displayed in decimal.
 *
 *    The strlen() of the returned string depends on the values in
 *    'ip_flags'.  For help in setting the column width of the output,
 *    use skipaddrCidrStringMaxlen() to get the length of the longest
 *    possible string.
 *
 *    See also skipaddrString().
 *
 *    Since SiLK 3.18.0.
 */
char *
skipaddrCidrString(
    char               *outbuf,
    const skipaddr_t   *ip,
    uint32_t            prefix,
    uint32_t            ip_flags);


/**
 *    Return the length of longest string expected to be returned by
 *    skipaddrString() when 'ip_flags' represent the flags value
 *    passed to that function and 'allow_ipv6' is 0 when only IPv4
 *    addresses are passed to skipaddrString() and non-zero otherwise.
 *
 *    The return value should only ever be used to set a column width.
 *    Always use a buffer having at least SKIPADDR_STRLEN characters
 *    as the 'outbuf' parameter.
 *
 *    This function ignores the SKIPADDR_UNMAP_V6 flag since the
 *    function does not know whether all IP addresses in this column
 *    fall into the ::ffff:0:0/96 netblock.  If the caller knows this,
 *    the caller may safely reduce the returned length by 22.  (The
 *    caller may reduce the length by 24 for SKIPADDR_CANONICAL,
 *    SKIPADDR_NO_MIXED, and SKIPADDR_HEXADECIMAL, and 22 for
 *    SKIPADDR_DECIMAL).
 *
 *    See also skipaddrCidrStringMaxlen().
 */
int
skipaddrStringMaxlen(
    unsigned int        allow_ipv6,
    uint32_t            ip_flags);


/**
 *    Return the length of longest string expected to be returned by
 *    skipaddrCidrString() when 'ip_flags' represent the flags value
 *    passed to that function and 'allow_ipv6' is 0 when only IPv4
 *    addresses are passed to skipaddrCidrString() and non-zero
 *    otherwise.
 *
 *    The return value should only ever be used to set a column width.
 *    Always use a buffer having at least SKIPADDR_CIDR_STRLEN
 *    characters as the 'outbuf' parameter.
 *
 *    This function treats SKIPADDR_UNMAP_V6 in the same way as
 *    skipaddrStringMaxlen(), which see.  When the input is known to
 *    contain only IPs in the ::ffff:0:0/96 netblock, the width may be
 *    reduced by 23 (or 25).
 *
 *    See also skipaddrStringMaxlen().
 *
 *    Since SiLK 3.18.0.
 */
int
skipaddrCidrStringMaxlen(
    unsigned int        allow_ipv6,
    uint32_t            ip_flags);


/**
 *    DEPRECATED.  Use skipaddrString() instead.
 *
 *    Converts the integer form of an IPv4 IP address to the dotted-quad
 *    version.  ip is taken to be in native byte order; returns a
 *    pointer to a static string buffer.
 *
 *    Returns NULL on error.
 *
 *    Replace with:
 *
 *    skipaddr_t ipaddr;
 *    char outbuf[SKIPADDR_STRLEN];
 *    skipaddrSetV4(&ipaddr, &ip);
 *    skipaddrString(outbuf, &ipaddr, 0);
 */
char *
num2dot(
    uint32_t            ip);


/**
 *    DEPRECATED.  Use skipaddrString() instead.
 *
 *    Like num2dot(), but will zero-pad the octects: num2dot0(0) will
 *    return "000.000.000.000".
 *
 *    Replace with:
 *
 *    skipaddr_t ipaddr;
 *    char outbuf[SKIPADDR_STRLEN];
 *    skipaddrSetV4(&ipaddr, &ip);
 *    skipaddrString(outbuf, &ipaddr, SKIPADDR_ZEROPAD);
 */
char *
num2dot0(
    uint32_t            ip)
    SK_GCC_DEPRECATED;


/**
 *    DEPRECATED.  Use skipaddrString() instead.
 *
 *    Thread safe version of num2dot().  The 'outbuf' should be at
 *    least SKIPADDR_STRLEN characters long.
 *
 *    Replace with:
 *
 *    skipaddr_t ipaddr;
 *    char outbuf[SKIPADDR_STRLEN];
 *    skipaddrSetV4(&ipaddr, &ip);
 *    skipaddrString(outbuf, &ipaddr, 0);
 */
char *
num2dot_r(
    uint32_t            ip,
    char               *outbuf);


/**
 *    DEPRECATED.  Use skipaddrString() instead.
 *
 *    Thread safe version of num2dot0().  The 'outbuf' must be at
 *    least SKIPADDR_STRLEN characters long.
 *
 *    Replace with:
 *
 *    skipaddr_t ipaddr;
 *    char outbuf[SKIPADDR_STRLEN];
 *    skipaddrSetV4(&ipaddr, &ip);
 *    skipaddrString(outbuf, &ipaddr, SKIPADDR_ZEROPAD);
 */
char *
num2dot0_r(
    uint32_t            ip,
    char               *outbuf)
    SK_GCC_DEPRECATED;


/**
 *    Strips all leading and trailing whitespace from 'line'.
 *    Modifies line in place.
 */
int
skStrip(
    char               *line);


/**
 *    Return a string describing the last error that occurred when
 *    invoking the skStringParse* functions.
 *
 *    The 'errcode' value is expected to be an silk_utils_errcode_t.
 */
const char *
skStringParseStrerror(
    int                 errcode);


/**
 *    Parses a C-string containing an IPv4 or IPv6 address in the
 *    "canonical" presentation form and sets the value pointed to by
 *    'out_val' to the result.
 *
 *    In addition, a single integer value will be parsed as IPv4
 *    address.
 *
 *    Returns 0 and puts the result into 'out_val' if parsing was
 *    successful and the 'ip_string' only contained the IP address and
 *    optional leading and/or trailing whitespace.
 *
 *    Returns a positive value and puts the result into 'out_val' if
 *    parsing was successful and 'ip_string' contains additional,
 *    non-whitespace text.  The return value is the number of
 *    characters that were parsed as part of the IP address and any
 *    leading whitespace.  That is, the return value is the position
 *    in 'ip_string' where the trailing text begins.  Whitespace
 *    between the IP address and the trailing text is not parsed.
 *
 *    Returns one of the silk_utils_errcode_t values on error.
 *
 *    This routine will not parse IPs in CIDR notation; or rather, it
 *    will parse the IP portion, but the return value will be the
 *    position of the '/' character.  To correctly parse IPs in CIDR
 *    notation, use skStringParseCIDR() or skStringParseIPWildcard().
 */
int
skStringParseIP(
    skipaddr_t         *out_val,
    const char         *ip_string);


/**
 *    Takes a C-string containing an IPv4 or IPv6 address and fills
 *    'ipwild' with the result.
 *
 *    Returns 0 and puts result into 'ipwild' if parsing was
 *    successful and 'ip_string' contained only the IP address and
 *    optional leading or trailing whitespace.
 *
 *    Returns one of the silk_utils_errcode_t values on error.
 *
 *    The 'ip_string' can be in CIDR notation such as "1.2.3.0/24" or
 *    "ff80::/16"; in the canonical form "1.2.3.4" or
 *    "::ffff:0102:0304", an integer 16909056, an integer with a CIDR
 *    designation 16909056/24, or in SiLK wildcard notation: a IP in
 *    the canonical form with an 'x' respresenting an entire octet
 *    "1.2.3.x" in IPv4 or entire hexadectet in IPv6
 *    "1:2:3:4:5:6:7.x", or a dotted quad with lists or ranges in any
 *    or all octets or hexadectets "1.2,3.4,5.6,7", "1.2.3.0-255",
 *    "::2-4", "1-2:3-4:5-6:7-8:9-a:b-c:d-e:0-ffff".
 *
 *    Note that wildcard characters (',','-','x') and CIDR notation
 *    cannot be combined in a single address.
 *
 *    See also skStringParseCIDR().
 */
int
skStringParseIPWildcard(
    skIPWildcard_t     *ipwild,
    const char         *ip_string);


/**
 *    Parses a C-string containing an IPv4 or IPv6 address in the
 *    "canonical" presentation form with an optional CIDR designation.
 *    Sets the values pointed to by 'out_val' and 'out_cidr' to the
 *    parsed IP address and the CIDR designation.  If no CIDR
 *    designation is present in 'ip_string', 'out_cidr' is set to the
 *    complete IP length: 32 for IPv4, 128 for IPv6.
 *
 *    In addition, a single integer value will be parsed as IPv4
 *    address, with an optional CIDR designation.
 *
 *    Returns 0 and puts the result into 'out_val' if parsing was
 *    successful and the 'ip_string' only contained the IP address,
 *    the CIDR designation, and optional leading and/or trailing
 *    whitespace.
 *
 *    Returns one of the silk_utils_errcode_t values on error.
 *
 *    If the CIDR mask is too large for the type of IP,
 *    SKUTILS_ERR_MAXIMUM is returned.
 *
 *    It is an error for 'ip_string' to contain any text other than
 *    the IP string, CIDR designation, and leading or trailing
 *    whitespace.  SKUTILS_ERR_BAD_CHAR will be returned.
 *
 *    This routine will not parse the SiLK IP Wildcard notation; use
 *    skStringParseIPWildcard() for that.
 */
int
skStringParseCIDR(
    skipaddr_t         *out_val,
    uint32_t           *out_cidr,
    const char         *ip_string);


/**
 *    A wrapper over skStringParseUint64() that uses unsiged 32-bit
 *    numbers unstead of 64-bit numbers.
 */
int
skStringParseUint32(
    uint32_t           *result_val,
    const char         *int_string,
    uint32_t            min_val,
    uint32_t            max_val);


/**
 *    Attempts to parse the C string 'int_string' as an unsigned
 *    64-bit decimal integer, and puts the result into the referent of
 *    'result_val'.  Ignores any whitespace before and after the
 *    number.
 *
 *    In addition, the function verifies that the value is between
 *    'min_val' and 'max_val' inclusive.  A 'max_val' of 0 is
 *    equivalent to UINT64_MAX; i.e., the largest value that will fit
 *    in a 64bit value.
 *
 *    Returns 0 and fills 'result_val' when 'int_string' contains only
 *    a number and leading or trailing whitespace and the number is
 *    parsable and the resulting value is within the limits.
 *
 *    Returns a positive value and fills 'result_val' when
 *    'int_string' contains a value within the limits and contains
 *    additional non-whitespace text after a parsed number.  The
 *    return value is the number of characters parsed in 'int_string'.
 *    For example, an 'int_string' of either "7x" or "7 x" causes the
 *    function to set the referent of 'result_val' to 7 and return 1.
 *
 *    Fills 'result_val' and returns either SKUTILS_ERR_MINIMUM or
 *    SKUTILS_ERR_MAXIMUM when the value is successfully parsed but is
 *    not within the limits.  When the value falls outside the
 *    possible range of the function, 'result_val' is unchanged and
 *    SKUTILS_ERR_OVERFLOW is returned.
 *
 *    Returns a silk_utils_errcode_t value on any other error and
 *    leaves 'result_val' unchanged.
 */
int
skStringParseUint64(
    uint64_t           *result_val,
    const char         *int_string,
    uint64_t            min_val,
    uint64_t            max_val);



/**
 *    Attempts to parse the C string 'dbl_string' as a floating point
 *    value (double).  In addition, verifies that the value is between
 *    'min_val' and 'max_val' inclusive.  A 'max_val' of 0 is
 *    equivalent to HUGE_VAL.  Puts the result into location pointed
 *    to by 'result_val'.  Ignores any whitespace around the number.
 *
 *    Returns 0 and fills 'result_val' when 'dbl_string' contains only
 *    a number and whitespace and the number is parsable and the
 *    resulting value is within the limits.
 *
 *    Returns a positive value and fills 'result_val' when
 *    'dbl_string' contains a value within the limits and contains
 *    additional non-whitespace text.  The return value is the number
 *    of characters parsed, not including whitespace between the
 *    number and the additional text; e.g; "1.1 x" would set
 *    *result_val to 1.1 and return 3.
 *
 *    Returns an silk_utils_errcode_t value on error.  Note that "NaN"
 *    returns SKUTILS_ERR_BAD_CHAR.
 */
int
skStringParseDouble(
    double             *result_val,
    const char         *dbl_string,
    double              min_val,
    double              max_val);

#ifdef __cplusplus
}
#endif
#endif /* _UTILS_H */

/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
